/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.servlets.admin.AdminConstants;
import fr.exemole.sctserver.servlets.admin.AdminResponseHandlerFactory;
import fr.exemole.sctserver.tools.EngineUtils;
import java.io.IOException;
import java.net.URL;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.html.PageHtmlPrinter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.Label;


/**
 *
 * @author Vincent Calame
 */
public abstract class AdminHtmlPage extends PageHtmlPrinter implements ResponseHandler {

    private final static HtmlAttributes MESSAGEDIV_ATTR = HA.classes("scrutari-admin-MessageArea");
    private final static HtmlAttributes ERROR_ATTR = HA.classes("scrutari-admin-ErrorMessage");
    private final static HtmlAttributes DONE_ATTR = HA.classes("scrutari-admin-DoneMessage");
    private final static HtmlAttributes TOOLDIV_ATTR = HA.classes("scrutari-admin-Tool");
    private final static HtmlAttributes CURRENT_ATTR = HA.classes("scrutari-admin-Current");
    private final static HtmlAttributes PANEL_ATTR = HA.classes("scrutari-admin-Panel");
    private final static HtmlAttributes BSCONTAINER_ATTR = HA.classes("container");
    private final static HtmlAttributes BSROW_ATTR = HA.classes("row");
    protected final SctEngine engine;
    protected final ScrutariSession scrutariSession;
    protected final Lang workingLang;
    protected final String title;
    private CommandMessage commandMessage;
    private short currentPage;

    public AdminHtmlPage(SctEngine engine, Lang workingLang) {
        this.engine = engine;
        this.scrutariSession = engine.getScrutariSession();
        this.workingLang = workingLang;
        addMessageLocalisation(EngineUtils.getMessageLocalisation(engine, workingLang));
        String resourcesPath = "../resources/";
        addJsUrl(resourcesPath + "third-lib/jquery/jquery.js");
        addJsUrl(resourcesPath + "third-lib/bootstrap/js/bootstrap.min.js");
        addCssUrl(resourcesPath + "third-lib/bootstrap/css/bootstrap.css");
        addCssUrl(resourcesPath + "third-lib/bootstrap/css/bootstrap-theme.css");
        addCssUrl(resourcesPath + "theme/css/bootstrap.css");
        addCssUrl(resourcesPath + "theme/css/admin.css");
        Label titleLabel = EngineUtils.getTitleLabel(engine, workingLang);
        if (titleLabel != null) {
            title = " – " + titleLabel.getLabelString();
        } else {
            title = "";
        }
        setTitle("Scrutari – [" + engine.getEngineName() + "]" + title);
        URL iconURL = engine.getEngineMetadata().getIcon();
        if (iconURL != null) {
            setIconPng(iconURL.toString());
        } else {
            setIconPng(resourcesPath + "theme/images/icon.png");
        }

    }

    public void setCurrentPage(short currentPage) {
        this.currentPage = currentPage;
    }

    public void setCommandMessage(CommandMessage commandMessage) {
        this.commandMessage = commandMessage;
    }

    @Override
    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    protected void initResponse(HttpServletResponse response) throws IOException {
        response.setContentType("text/html;charset=UTF-8");
        initPrinter(response.getWriter());

    }

    protected void printTools() {
        startBSContainer();
        startBSRow();
        this
                .H1()
                .escape("Scrutari – ")
                .CODE()
                .escape("[")
                .escape(engine.getEngineName())
                .escape("]")
                ._CODE()
                .escape(title)
                ._H1();
        endBSRow();
        startBSRow();
        this
                .DIV(TOOLDIV_ATTR);
        printToolLink(AdminConstants.INDEX_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.STATS_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.FICHES_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.MOTSCLES_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.LEXIES_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.SEARCHLOGS_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.SCRUTARIJS_PAGE);
        this
                ._DIV();
        endBSRow();
        endBSContainer();

    }

    protected void printMessage() {
        if (commandMessage != null) {
            startBSRow();
            this
                    .DIV(MESSAGEDIV_ATTR);
            HtmlPrinterUtils.printCommandMessage(this, commandMessage, DONE_ATTR, ERROR_ATTR);
            this
                    ._DIV();
            endBSRow();
        }
    }

    protected void start() {
        start(workingLang);
    }

    protected void startBSContainer() {
        this
                .DIV(BSCONTAINER_ATTR);
    }

    protected void endBSContainer() {
        this
                ._DIV();
    }

    protected void startBSRow() {
        this
                .DIV(BSROW_ATTR);
    }

    protected void endBSRow() {
        this
                ._DIV();
    }

    protected void startPanel() {
        this
                .DIV(PANEL_ATTR);
    }

    protected void endPanel() {
        this
                ._DIV();
    }


    private void printToolSeparator() {
        this
                .escape(" — ");
    }

    private void printToolLink(short page) {
        String key = AdminResponseHandlerFactory.pageToString(page);
        if (currentPage == page) {
            this
                    .SPAN(CURRENT_ATTR);
        } else {
            this
                    .A(HA.href(key + ".html"));
        }
        this
                .localize(AdminResponseHandlerFactory.getPageMessageKey(page));
        if (currentPage == page) {
            this
                    ._SPAN();
        } else {
            this
                    ._A();
        }
    }

    public SctEngine getEngine() {
        return engine;
    }

}
