/* ScrutariLib_Data - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.net.URI;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.html.jsoup.TrustedHtmlFactories;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;
import net.scrutari.datauri.BaseURI;


/**
 *
 * @author Vincent Calame
 */
public final class BaseData implements Serializable {

    public final static short INTITULE_SHORT = 1;
    public final static short INTITULE_LONG = 2;
    private final BaseURI baseURI;
    private final int codebase;
    private final String scrutariSourceName;
    private final Lang[] langsUI;
    private final String baseIconPart1;
    private final String baseIconPart2;
    private final BaseLangData[] baseLangDataArray;
    private final int[] codesCorpus;
    private final int[] codesThesaurus;
    private final AttributeHolder attributeHolder;

    private BaseData(BaseURI baseURI, int codebase, String scrutariSourceName, Lang[] langsUI, String baseIconPart1, String baseIconPart2, BaseLangData[] baseLangDataArray, int[] codesCorpus, int[] codesThesaurus, AttributeHolder attributeHolder) {
        this.baseURI = baseURI;
        this.codebase = codebase;
        this.scrutariSourceName = scrutariSourceName;
        this.langsUI = langsUI;
        this.baseIconPart1 = baseIconPart1;
        this.baseIconPart2 = baseIconPart2;
        this.baseLangDataArray = baseLangDataArray;
        this.codesCorpus = codesCorpus;
        this.codesThesaurus = codesThesaurus;
        this.attributeHolder = attributeHolder;
    }

    public String getScrutariSourceName() {
        return scrutariSourceName;
    }

    public BaseURI getBaseURI() {
        return baseURI;
    }

    public int getCodeBase() {
        return codebase;
    }

    public int getLangUiCount() {
        return langsUI.length;
    }

    public Lang getLangUi(int i) {
        return langsUI[i];
    }

    public Lang checkLangUi(Lang lang) {
        int length = langsUI.length;
        if (length == 0) {
            return lang;
        }
        for (int i = 0; i < length; i++) {
            if (lang.equals(langsUI[i])) {
                return lang;
            }
        }
        if (!lang.isRootLang()) {
            Lang rootLang = lang.getRootLang();
            for (int i = 0; i < length; i++) {
                if (rootLang.equals(langsUI[i])) {
                    return rootLang;
                }
            }
        }
        return langsUI[0];
    }

    public String getBaseIcon(Lang langUi) {
        if (baseIconPart2 == null) {
            return baseIconPart1;
        }
        StringBuilder buf = new StringBuilder(baseIconPart1.length() + baseIconPart2.length() + 10);
        buf.append(baseIconPart1);
        buf.append(langUi.toString());
        buf.append(baseIconPart2);
        return buf.toString();
    }

    public BaseLangData getLangDataByLang(Lang lang) {
        int length = baseLangDataArray.length;
        for (int i = 0; i < length; i++) {
            BaseLangData langData = baseLangDataArray[i];
            if (langData.lang.equals(lang)) {
                return langData;
            }
        }
        return null;
    }

    public int getLangDataCount() {
        return baseLangDataArray.length;
    }

    public BaseLangData getLangDataByIndex(int i) {
        return baseLangDataArray[i];
    }

    public int getCorpusCount() {
        return codesCorpus.length;
    }

    public int getCodeCorpus(int i) {
        return codesCorpus[i];
    }

    public int getThesaurusCount() {
        return codesThesaurus.length;
    }

    public int getCodeThesaurus(int i) {
        return codesThesaurus[i];
    }

    public AttributeHolder getAttributHolder() {
        return attributeHolder;
    }

    public static BaseData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        BaseURI baseURI = (BaseURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codebase = primitivesReader.readInt();
        String scrutariSourceName = primitivesReader.readString();
        int langsUILength = primitivesReader.readInt();
        Lang[] langsUI = new Lang[langsUILength];
        for (int i = 0; i < langsUILength; i++) {
            langsUI[i] = Lang.build(primitivesReader.readString());
        }
        String baseIconPart1 = primitivesReader.readString();
        String baseIconPart2 = primitivesReader.readString();
        int langDataLength = primitivesReader.readInt();
        BaseLangData[] baseLangDataArray = new BaseLangData[langDataLength];
        for (int i = 0; i < langDataLength; i++) {
            Lang lang = Lang.build(primitivesReader.readString());
            BaseLangData baseLangData = new BaseLangData(lang);
            readLabel(baseLangData, primitivesReader);
            readLabel(baseLangData, primitivesReader);
            baseLangDataArray[i] = baseLangData;
        }
        int corpusLength = primitivesReader.readInt();
        int[] codesCorpus = new int[corpusLength];
        for (int i = 0; i < corpusLength; i++) {
            codesCorpus[i] = primitivesReader.readInt();
        }
        int thesaurusLength = primitivesReader.readInt();
        int[] codesThesaurus = new int[thesaurusLength];
        for (int i = 0; i < thesaurusLength; i++) {
            codesThesaurus[i] = primitivesReader.readInt();
        }
        AttributeHolder attributeHolder = ScrutariAttributes.fromPrimitives(primitivesReader);
        return new BaseData(baseURI, codebase, scrutariSourceName, langsUI, baseIconPart1, baseIconPart2, baseLangDataArray, codesCorpus, codesThesaurus, attributeHolder);
    }

    public static void toPrimitives(BaseData baseData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(baseData.baseURI, primitivesWriter);
        primitivesWriter.writeInt(baseData.codebase);
        primitivesWriter.writeString(baseData.scrutariSourceName);
        int langsUILength = baseData.langsUI.length;
        primitivesWriter.writeInt(langsUILength);
        for (int i = 0; i < langsUILength; i++) {
            primitivesWriter.writeString(baseData.langsUI[i].toString());
        }
        primitivesWriter.writeString(baseData.baseIconPart1);
        primitivesWriter.writeString(baseData.baseIconPart2);
        int langDataLength = baseData.baseLangDataArray.length;
        primitivesWriter.writeInt(langDataLength);
        for (int i = 0; i < langDataLength; i++) {
            BaseLangData baseLangData = baseData.baseLangDataArray[i];
            primitivesWriter.writeString(baseLangData.lang.toString());
            writeLabel(INTITULE_SHORT, baseLangData, primitivesWriter);
            writeLabel(INTITULE_LONG, baseLangData, primitivesWriter);
        }
        int corpusLength = baseData.codesCorpus.length;
        primitivesWriter.writeInt(corpusLength);
        for (int i = 0; i < corpusLength; i++) {
            primitivesWriter.writeInt(baseData.codesCorpus[i]);
        }
        int thesaurusLength = baseData.codesThesaurus.length;
        primitivesWriter.writeInt(thesaurusLength);
        for (int i = 0; i < thesaurusLength; i++) {
            primitivesWriter.writeInt(baseData.codesThesaurus[i]);
        }
        ScrutariAttributes.toPrimitives(baseData.attributeHolder, primitivesWriter);
    }

    private static void writeLabel(short labelType, BaseLangData baseLangData, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeShort(labelType);
        String labelString = baseLangData.getLabelString(labelType);
        primitivesWriter.writeString(labelString);
    }

    private static void readLabel(BaseLangData baseLangData, PrimitivesReader primitivesReader) throws IOException {
        short type = primitivesReader.readShort();
        String labelString = primitivesReader.readString();
        if (labelString != null) {
            baseLangData.setLabel(type, DataLabelBuilder.toLabel(baseLangData.lang, labelString));
        }
    }


    public static class BaseLangData {

        private final Lang lang;
        private Label shortLabel;
        private Label longLabel;

        private BaseLangData(Lang lang) {
            this.lang = lang;
        }

        public Label getLabel(short type) {
            switch (type) {
                case INTITULE_SHORT:
                    return shortLabel;
                case INTITULE_LONG:
                    return longLabel;
            }
            return null;
        }

        public String getLabelString(short type) {
            Label label = getLabel(type);
            if (label == null) {
                return null;
            }
            return label.getLabelString();
        }

        public Lang getLang() {
            return lang;
        }

        private void setLabel(short type, Label label) {
            switch (type) {
                case INTITULE_SHORT:
                    shortLabel = label;
                    break;
                case INTITULE_LONG:
                    longLabel = label;
                    break;
            }
        }

    }


    public static class Builder {

        private final BaseURI baseURI;
        private final List<Integer> corpusList = new ArrayList<Integer>();
        private final List<Integer> thesaurusList = new ArrayList<Integer>();
        private final Map<Lang, BaseLangData> baseLangDataMap = new LinkedHashMap<Lang, BaseLangData>();
        private final Set<Lang> langsUiSet = new LinkedHashSet<Lang>();
        private String baseIcon;
        private AppendableAttributeHolderBuilder attributeHolderBuilder;

        public Builder(BaseURI baseURI) {
            this.baseURI = baseURI;
        }

        public void addLabel(short type, Label label) {
            label = DataLabelBuilder.checkLabel(label);
            if (label == null) {
                return;
            }
            Lang lang = label.getLang();
            BaseLangData langData = baseLangDataMap.get(lang);
            if (langData == null) {
                langData = new BaseLangData(lang);
                baseLangDataMap.put(lang, langData);
            }
            langData.setLabel(type, label);
        }

        public void addLangUI(Lang lang) {
            langsUiSet.add(lang);
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

        public boolean hasBaseIcon() {
            return (this.baseIcon != null);
        }

        public boolean setBaseIcon(URI baseIconURI) {
            if (baseIconURI == null) {
                this.baseIcon = null;
                return true;
            }
            if (!baseIconURI.isAbsolute()) {
                return false;
            }
            if (baseIconURI.isOpaque()) {
                return false;
            }
            String scheme = baseIconURI.getScheme();
            if ((!scheme.equals("http")) && (!scheme.equals("https"))) {
                return false;
            }
            this.baseIcon = baseIconURI.toASCIIString();
            return true;
        }

        public void addCodeCorpus(int codecorpus) {
            corpusList.add(codecorpus);
        }

        public void addCodeThesaurus(int codethesaurus) {
            thesaurusList.add(codethesaurus);
        }

        public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
            if (attributeHolderBuilder == null) {
                attributeHolderBuilder = new AppendableAttributeHolderBuilder();
            }
            List<CleanedString> filteredList = new ArrayList<CleanedString>();
            for (CleanedString cleanedString : valueList) {
                String strg = cleanedString.toString();
                String checked = TrustedHtmlFactories.TEXT.check(strg);
                if (checked.equals(strg)) {
                    filteredList.add(cleanedString);
                } else {
                    cleanedString = CleanedString.newInstance(checked);
                    if (cleanedString != null) {
                        filteredList.add(cleanedString);
                    }
                }
            }
            attributeHolderBuilder.addAttribute(attributeKey, filteredList);
        }

        public BaseData toBaseData(int codebase, String scrutariSourceName) {
            int corpusCount = corpusList.size();
            int[] codesCorpus = new int[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                codesCorpus[i] = corpusList.get(i);
            }
            int thesaurusCount = thesaurusList.size();
            int[] codesThesaurus = new int[thesaurusCount];
            for (int i = 0; i < thesaurusCount; i++) {
                codesThesaurus[i] = thesaurusList.get(i);
            }
            Lang[] langsUI = langsUiSet.toArray(new Lang[langsUiSet.size()]);
            String baseIconPart1 = null;
            String baseIconPart2 = null;
            if (baseIcon != null) {
                int idx = baseIcon.indexOf("$LANGUI");
                if (idx == -1) {
                    baseIconPart1 = baseIcon;
                } else {
                    baseIconPart1 = baseIcon.substring(0, idx);
                    baseIconPart2 = baseIcon.substring(idx + 7);
                }
            }
            BaseLangData[] baseLangDataArray = baseLangDataMap.values().toArray(new BaseLangData[baseLangDataMap.size()]);
            AttributeHolder attributeHolder = (attributeHolderBuilder != null) ? attributeHolderBuilder.toAttributeHolder() : AttributeUtils.EMPTY_ATTRIBUTEHOLDER;
            return new BaseData(baseURI, codebase, scrutariSourceName, langsUI, baseIconPart1, baseIconPart2, baseLangDataArray, codesCorpus, codesThesaurus, attributeHolder);
        }

    }

}
