/* SctEngine - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.json.BaseDataJson;
import fr.exemole.sctserver.json.CorpusDataJson;
import fr.exemole.sctserver.json.JsonUtils;
import fr.exemole.sctserver.json.ThesaurusDataJson;
import fr.exemole.sctserver.json.category.CategoryJson_A;
import fr.exemole.sctserver.json.category.CategoryJson_B;
import fr.exemole.sctserver.json.enginemetadata.EngineMetadataJson_A;
import fr.exemole.sctserver.json.enginemetadata.EngineMetadataJson_B;
import fr.exemole.sctserver.request.DataFilter;
import fr.exemole.sctserver.tools.EngineUtils;
import java.io.IOException;
import java.util.List;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.stats.FicheStats;
import net.scrutari.db.api.stats.ScrutariDBStats;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public class EngineInfoJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariSession scrutariSession;
    private final Lang lang;
    private final DataFilter dataFilter;

    public EngineInfoJsonProducer(int version, ScrutariSession scrutariSession, Lang lang, DataFilter dataFilter) {
        if (lang == null) {
            throw new IllegalArgumentException("lang is null");
        }
        this.version = version;
        this.scrutariSession = scrutariSession;
        this.lang = lang;
        this.dataFilter = dataFilter;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        if (version > 1) {
            printVersion2(jsonWriter);
        } else {
            printVersion1(jsonWriter);
        }
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
    }

    private void printVersion2(JSONWriter jsonWriter) throws IOException {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        SctEngine engine = scrutariSession.getEngine();
        GlobalSearchOptions globalSearchOptions = scrutariSession.getGlobalSearchOptions();
        MessageLocalisation messageLocalisation = EngineUtils.getMessageLocalisation(scrutariSession.getEngine(), lang);
        DataAccess dataAccess = scrutariDB.openDataAccess();
        FicheStats engineFicheStats = scrutariDB.getStats().getEngineFicheStats();
        ScrutariDBStats stats;
        if (dataFilter.isWithStats()) {
            stats = scrutariDB.getStats();
        } else {
            stats = null;
        }
        jsonWriter.key("engineInfo");
        jsonWriter.object();
        if (dataFilter.isWithBase()) {
            jsonWriter.key("baseMap");
            jsonWriter.object();
            int baseCount = dataAccess.getBaseCount();
            for (int i = 0; i < baseCount; i++) {
                BaseData baseData = dataAccess.getBaseDataByIndex(i);
                jsonWriter.key("code_" + baseData.getCodeBase());
                BaseDataJson.object(jsonWriter, baseData, lang, stats);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithCorpus()) {
            jsonWriter.key("corpusMap");
            jsonWriter.object();
            int count = dataAccess.getCorpusCount();
            for (int i = 0; i < count; i++) {
                CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                jsonWriter.key("code_" + corpusData.getCodeCorpus());
                CorpusDataJson.object(jsonWriter, corpusData, lang, stats);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithThesaurus()) {
            jsonWriter.key("thesaurusMap");
            jsonWriter.object();
            int count = dataAccess.getThesaurusCount();
            for (int i = 0; i < count; i++) {
                ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
                jsonWriter.key("code_" + thesaurusData.getCodeThesaurus());
                ThesaurusDataJson.object(jsonWriter, thesaurusData, lang, stats);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithStats()) {
            jsonWriter.key("stats");
            jsonWriter.object();
            JsonUtils.addCountStats(jsonWriter, stats.getEngineCountStats());
            JsonUtils.addFicheStats(jsonWriter, engineFicheStats);
            jsonWriter.endObject();
        }
        if (dataFilter.isWithCategory()) {
            List<Category> categoryList = globalSearchOptions.getCategoryList();
            if (!categoryList.isEmpty()) {
                jsonWriter.key("categoryMap");
                jsonWriter.object();
                for (Category category : categoryList) {
                    jsonWriter.key(category.getCategoryDef().getName());
                    CategoryJson_B.object(jsonWriter, category, lang, dataFilter.isWithStats());
                }
                jsonWriter.endObject();
            }
        }
        if (dataFilter.isWithLang()) {
            int count = engineFicheStats.getLangCount();
            jsonWriter.key("langMap");
            jsonWriter.object();
            for (int i = 0; i < count; i++) {
                Lang statLang = engineFicheStats.getLang(i);
                String code = statLang.toString();
                jsonWriter.key(code);
                String value = messageLocalisation.toString(code);
                if (value != null) {
                    value = StringUtils.getFirstPart(value);
                } else {
                    value = code;
                }
                jsonWriter.value(value);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithAttributes()) {
            JsonUtils.addAttributes(jsonWriter, scrutariDB.getAttributeKeyCoder(), lang, version);
        }
        if (dataFilter.isWithMetadata()) {
            jsonWriter.key("metadata");
            jsonWriter.object();
            EngineMetadataJson_B.properties(jsonWriter, engine.getEngineName(), engine.getEngineMetadata(), lang, false);
            jsonWriter.endObject();
        }
        jsonWriter.endObject();
        dataAccess.close();
    }

    private void printVersion1(JSONWriter jsonWriter) throws IOException {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        SctEngine engine = scrutariSession.getEngine();
        GlobalSearchOptions globalSearchOptions = scrutariSession.getGlobalSearchOptions();
        MessageLocalisation messageLocalisation = EngineUtils.getMessageLocalisation(scrutariSession.getEngine(), lang);
        DataAccess dataAccess = scrutariDB.openDataAccess();
        FicheStats engineFicheStats = scrutariDB.getStats().getEngineFicheStats();
        ScrutariDBStats stats;
        if (dataFilter.isWithStats()) {
            stats = scrutariDB.getStats();
        } else {
            stats = null;
        }
        jsonWriter.key("engineInfo");
        jsonWriter.object();
        if (dataFilter.isWithBase()) {
            jsonWriter.key("baseMap");
            jsonWriter.object();
            int baseCount = dataAccess.getBaseCount();
            for (int i = 0; i < baseCount; i++) {
                BaseData baseData = dataAccess.getBaseDataByIndex(i);
                jsonWriter.key("code_" + baseData.getCodeBase());
                BaseDataJson.object(jsonWriter, baseData, lang, stats);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithCorpus()) {
            jsonWriter.key("corpusMap");
            jsonWriter.object();
            int count = dataAccess.getCorpusCount();
            for (int i = 0; i < count; i++) {
                CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                jsonWriter.key("code_" + corpusData.getCodeCorpus());
                CorpusDataJson.object(jsonWriter, corpusData, lang, stats);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithThesaurus()) {
            jsonWriter.key("thesaurusMap");
            jsonWriter.object();
            int count = dataAccess.getThesaurusCount();
            for (int i = 0; i < count; i++) {
                ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
                jsonWriter.key("code_" + thesaurusData.getCodeThesaurus());
                ThesaurusDataJson.object(jsonWriter, thesaurusData, lang, stats);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithStats()) {
            jsonWriter.key("stats");
            jsonWriter.object();
            JsonUtils.addCountStats(jsonWriter, stats.getEngineCountStats());
            JsonUtils.addFicheStats(jsonWriter, engineFicheStats);
            jsonWriter.endObject();
        }
        if (dataFilter.isWithCategory()) {
            List<Category> categoryList = globalSearchOptions.getCategoryList();
            if (!categoryList.isEmpty()) {
                jsonWriter.key("categoryMap");
                jsonWriter.object();
                for (Category category : categoryList) {
                    jsonWriter.key(category.getCategoryDef().getName());
                    CategoryJson_A.object(jsonWriter, category, lang, dataFilter.isWithStats());
                }
                jsonWriter.endObject();
            }
        }
        if (dataFilter.isWithLang()) {
            int count = engineFicheStats.getLangCount();
            jsonWriter.key("langMap");
            jsonWriter.object();
            for (int i = 0; i < count; i++) {
                Lang lang = engineFicheStats.getLang(i);
                String code = lang.toString();
                jsonWriter.key(code);
                String value = messageLocalisation.toString(code);
                if (value != null) {
                    value = StringUtils.getFirstPart(value);
                } else {
                    value = code;
                }
                jsonWriter.value(value);
            }
            jsonWriter.endObject();
        }
        if (dataFilter.isWithAttributes()) {
            JsonUtils.addAttributes(jsonWriter, scrutariDB.getAttributeKeyCoder(), lang, version);
        }
        if (dataFilter.isWithMetadata()) {
            jsonWriter.key("metadata");
            jsonWriter.object();
            EngineMetadataJson_A.properties(jsonWriter, engine.getEngineName(), engine.getEngineMetadata(), lang, false);
            jsonWriter.endObject();
        }
        jsonWriter.endObject();
        dataAccess.close();
    }

}
