/* ScrutariLib_SearchEngine - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.CheckedKeyString;
import net.mapeadores.util.text.CleanedString;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.ListReduction;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class BuildingUtils {

    public final static CheckedKeyString CATEGORY_NAMESPACE = CheckedKeyString.build("category");
    public final static AttributeKey CATEGORY_LIST_KEY = AttributeKey.build(CATEGORY_NAMESPACE, "list");
    public final static AttributeKey CATEGORY_MODE_KEY = AttributeKey.build(CATEGORY_NAMESPACE, "mode");
    public final static CleanedString EXCLUDE_MODE = CleanedString.newInstance("mode", true);
    private final static Set<Integer> EMPTY_SET = Collections.emptySet();

    private BuildingUtils() {

    }

    public static void computeReduction(SearchOptionsBuilder searchOptionsBuilder, DataAccess dataAccess, GlobalSearchOptions globalSearchOptions) {
        SearchOptionsDef searchOptionsDef = searchOptionsBuilder.getSearchOptionsDef();
        ListReduction ficheListReduction = searchOptionsDef.getListReduction(ScrutariDataURI.FICHEURI_TYPE);
        if (ficheListReduction != null) {
            Map<Integer, Integer> elementMap = new LinkedHashMap<Integer, Integer>();
            for (ScrutariDataURI uri : ficheListReduction.getScrutariDataURIList()) {
                int code = dataAccess.getCode(uri);
                if (code > 0) {
                    if (!elementMap.containsKey(code)) {
                        FicheData ficheData = dataAccess.getFicheDataByCode(code);
                        elementMap.put(ficheData.getCodeFiche(), ficheData.getCodeCorpus());
                    }
                }
            }
            searchOptionsBuilder.reduceFicheCodes(CodesUtils.toElementCodes(elementMap));
        }
        Codes corpusCodes = computeCorpusCodeList(dataAccess, searchOptionsDef);
        if (corpusCodes != null) {
            searchOptionsBuilder.reduceCorpusCodes(corpusCodes);
        }
        Codes corpusCategoryCodes = computeCategory(dataAccess, searchOptionsDef, globalSearchOptions);
        if (corpusCategoryCodes != null) {
            searchOptionsBuilder.reduceCorpusCodes(corpusCategoryCodes);
        }
        ListReduction thesaurusListReduction = searchOptionsDef.getListReduction(ScrutariDataURI.THESAURUSURI_TYPE);
        if (thesaurusListReduction != null) {
            Set<Integer> thesaurusSet = toSet(dataAccess, thesaurusListReduction);
            Codes thesaurusCodes;
            if (thesaurusListReduction.isExclude()) {
                thesaurusCodes = excludeThesaurus(dataAccess, thesaurusSet);
            } else {
                thesaurusCodes = CodesUtils.toCodes(thesaurusSet);
            }
            searchOptionsBuilder.reduceThesaurusCodes(thesaurusCodes);
        }
    }

    private static Codes computeCategory(DataAccess dataAccess, SearchOptionsDef searchOptionsDef, GlobalSearchOptions globalSearchOptions) {
        Attribute attribute = searchOptionsDef.getAttributeByKey(CATEGORY_LIST_KEY);
        if (attribute == null) {
            return null;
        }
        boolean exclude = false;
        Attribute modeAttribute = searchOptionsDef.getAttributeByKey(CATEGORY_MODE_KEY);
        if (modeAttribute != null) {
            exclude = modeAttribute.getValue(0).equals(EXCLUDE_MODE.toString());
        }
        int categoryCount = attribute.getValueCount();
        List<Integer> selectionList = new ArrayList<Integer>();
        if (exclude) {
            Set<Integer> excludeSet = new HashSet<Integer>();
            for (int i = 0; i < categoryCount; i++) {
                String categoryName = attribute.getValue(i);
                Category category = globalSearchOptions.getCategoryByName(categoryName);
                if (category != null) {
                    Codes codes = category.getCorpusCodes();
                    int codeCount = codes.getCodeCount();
                    for (int j = 0; j < codeCount; j++) {
                        excludeSet.add(codes.getCode(j));
                    }
                }
            }
            int corpusCount = dataAccess.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                if (!excludeSet.contains(corpusData.getCodeCorpus())) {
                    selectionList.add(corpusData.getCodeCorpus());
                }
            }
        } else {
            for (int i = 0; i < categoryCount; i++) {
                String categoryName = attribute.getValue(i);
                Category category = globalSearchOptions.getCategoryByName(categoryName);
                if (category != null) {
                    Codes codes = category.getCorpusCodes();
                    int codeCount = codes.getCodeCount();
                    for (int j = 0; j < codeCount; j++) {
                        selectionList.add(codes.getCode(j));
                    }
                }
            }
        }
        return CodesUtils.toCodes(selectionList);
    }

    private static Codes computeCorpusCodeList(DataAccess dataAccess, SearchOptionsDef searchOptionsDef) {
        ListReduction baseListReduction = searchOptionsDef.getListReduction(ScrutariDataURI.BASEURI_TYPE);
        ListReduction corpusListReduction = searchOptionsDef.getListReduction(ScrutariDataURI.CORPUSURI_TYPE);
        boolean withBaseReduction = (baseListReduction != null);
        boolean withCorpusReduction = (corpusListReduction != null);
        if (!withBaseReduction && !withCorpusReduction) {
            return null;
        }
        Codes corpusCodes;
        if ((!withBaseReduction) && (!corpusListReduction.isExclude())) {
            corpusCodes = CodesUtils.toCodes(toSet(dataAccess, corpusListReduction));
        } else {
            Set<Integer> baseSet = null;
            boolean baseExclude = false;
            Set<Integer> corpusSet = null;
            boolean corpusExclude = false;
            if (withBaseReduction) {
                baseSet = toSet(dataAccess, baseListReduction);
                baseExclude = baseListReduction.isExclude();
            }
            if (withCorpusReduction) {
                corpusSet = toSet(dataAccess, corpusListReduction);
                corpusExclude = corpusListReduction.isExclude();
            }
            CorpusFilter corpusFilter = new CorpusFilter(baseSet, baseExclude, corpusSet, corpusExclude);
            List<Integer> resultList = new ArrayList<Integer>();
            int corpusCount = dataAccess.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                if (corpusFilter.accept(corpusData)) {
                    resultList.add(corpusData.getCodeCorpus());
                }
            }
            corpusCodes = CodesUtils.toCodes(resultList);
        }
        return corpusCodes;
    }


    private static Codes excludeThesaurus(DataAccess dataAccess, Set<Integer> excludeThesaurusSet) {
        List<Integer> resultList = new ArrayList<Integer>();
        int thesaurusCount = dataAccess.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
            int codethesaurus = thesaurusData.getCodeThesaurus();
            if (!excludeThesaurusSet.contains(codethesaurus)) {
                resultList.add(codethesaurus);
            }
        }
        return CodesUtils.toCodes(resultList);
    }

    private static Set<Integer> toSet(DataAccess dataAccess, ListReduction listReduction) {
        List<ScrutariDataURI> uriList = listReduction.getScrutariDataURIList();
        if (uriList.isEmpty()) {
            return EMPTY_SET;
        }
        Set<Integer> set = new HashSet<Integer>();
        for (ScrutariDataURI uri : uriList) {
            int code = dataAccess.getCode(uri);
            if (code > 0) {
                set.add(code);
            }
        }
        return set;
    }


    private static class CorpusFilter {

        private final Set<Integer> baseSet;
        private final Set<Integer> corpusSet;
        private final boolean baseExclude;
        private final boolean corpusExclude;

        private CorpusFilter(Set<Integer> baseSet, boolean baseExclude, Set<Integer> corpusSet, boolean corpusExclude) {
            this.baseSet = baseSet;
            if (baseSet == null) {
                this.baseExclude = false;
            } else {
                this.baseExclude = baseExclude;
            }
            this.corpusSet = corpusSet;
            if (corpusSet == null) {
                this.corpusExclude = false;
            } else {
                this.corpusExclude = corpusExclude;
            }
        }

        private boolean accept(CorpusData corpusData) {
            int codebase = corpusData.getCodeBase();
            int codecorpus = corpusData.getCodeCorpus();
            boolean baseAccept = false;
            if (baseSet != null) {
                if (baseExclude) {
                    if (baseSet.contains(codebase)) {
                        return false;
                    } else {
                        baseAccept = true;
                    }
                } else if (baseSet.contains(codebase)) {
                    baseAccept = true;
                }
            }
            boolean corpusAccept = false;
            if (corpusSet != null) {
                if (corpusExclude) {
                    if (corpusSet.contains(codecorpus)) {
                        return false;
                    } else {
                        corpusAccept = true;
                    }
                } else if (corpusSet.contains(codecorpus)) {
                    corpusAccept = true;
                }
            }
            return (baseAccept || corpusAccept);
        }

    }

}
