/* SctServer - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.api.EngineMetadata;
import fr.exemole.sctserver.tools.ErrorUtils;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.ChangeLabelHolderBuilder;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.Phrases;
import net.mapeadores.util.text.PhrasesBuilder;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class MetadataDOMReader {

    private final ChangeLabelHolderBuilder titleLabelHolderBuilder = new ChangeLabelHolderBuilder();
    private final PhrasesBuilder phrasesBuilder = new PhrasesBuilder();
    private final AppendableAttributeHolderBuilder attributeHolderBuilder = new AppendableAttributeHolderBuilder();
    private final MessageHandler messageHandler;
    private URL iconUrl;
    private URL websiteUrl;
    private Lang defaultLang;

    public MetadataDOMReader(MessageHandler messageHandler) {
        this.messageHandler = messageHandler;
    }

    public EngineMetadata readMetadata(Element element, String xpath) {
        RootHandler rootHandler = new RootHandler(xpath + "/" + element.getTagName());
        DOMUtils.readChildren(element, rootHandler);
        return new InternalEngineMetadata(titleLabelHolderBuilder.toLabelHolder(), phrasesBuilder.toPhrases(), iconUrl, websiteUrl, defaultLang, attributeHolderBuilder.toAttributeHolder());
    }


    private class RootHandler implements ElementHandler {

        private final String xpath;

        private RootHandler(String xpath) {
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("label")) {
                try {
                    Label label = LabelUtils.readLabel(element);
                    if (label != null) {
                        titleLabelHolderBuilder.putLabel(label);
                    }
                } catch (ParseException pe) {
                    ErrorUtils.severe(messageHandler, "_ error.wrong.xml.langattributevalue", xpath + "/" + tagName, element.getAttribute("xml:lang"));
                }
            } else if (tagName.equals("title")) {
                LabelHolderHandler handler = new LabelHolderHandler(titleLabelHolderBuilder, xpath + "/" + tagName);
                DOMUtils.readChildren(element, handler);
            } else if (tagName.equals("phrase")) {
                ConfDOMUtils.readPhraseElement(phrasesBuilder, element, messageHandler, xpath + "/" + tagName);
            } else if (tagName.equals("description")) {
                ConfDOMUtils.readPhraseElement(phrasesBuilder, element, messageHandler, xpath + "/" + tagName, "description");
            } else if (tagName.equals("icon")) {
                String data = XMLUtils.getData(element);
                if (data.length() > 0) {
                    iconUrl = getURL(data, xpath + "/" + tagName);
                }
            } else if (tagName.equals("website")) {
                String data = XMLUtils.getData(element);
                if (data.length() > 0) {
                    websiteUrl = getURL(data, xpath + "/" + tagName);
                }
            } else if ((tagName.equals("default-lang")) || (tagName.equals("default-lang-ui"))) {
                String data = XMLUtils.getData(element);
                if (data.length() > 0) {
                    try {
                        defaultLang = Lang.parse(data);
                    } catch (ParseException pe) {
                        ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue", xpath + tagName, data);
                    }
                }
            } else if (tagName.equals("attr")) {
                AttributeUtils.readAttrElement(attributeHolderBuilder, element);
            }
        }

        private URL getURL(String data, String urlXPath) {
            try {
                URL url = new URL(data);
                return url;
            } catch (MalformedURLException mue) {
                ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue_url", urlXPath, data, mue.getMessage());
                return null;
            }
        }

    }


    private class LabelHolderHandler implements ElementHandler {

        private final ChangeLabelHolderBuilder labelHolderBuilder;
        private final String xpath;


        private LabelHolderHandler(ChangeLabelHolderBuilder labelHolderBuilder, String xpath) {
            this.labelHolderBuilder = labelHolderBuilder;
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                try {
                    Label label = LabelUtils.readLabel(element);
                    if (label != null) {
                        labelHolderBuilder.putLabel(label);
                    }
                } catch (ParseException pe) {
                    ErrorUtils.severe(messageHandler, "_ error.wrong.xml.langattributevalue", xpath + "/" + tagName, element.getAttribute("xml:lang"));
                }
            }
        }

    }


    private static class InternalEngineMetadata implements EngineMetadata {

        private final LabelHolder titleLabelHolder;
        private final Phrases phrases;
        private final URL iconUrl;
        private final URL websiteUrl;
        private final Lang defaultLang;
        private final AttributeHolder attributeHolder;

        private InternalEngineMetadata(LabelHolder titleLabelHolder, Phrases phrases, URL iconUrl, URL websiteUrl, Lang defaultLang, AttributeHolder attributeHolder) {
            this.titleLabelHolder = titleLabelHolder;
            this.phrases = phrases;
            this.iconUrl = iconUrl;
            this.websiteUrl = websiteUrl;
            this.defaultLang = defaultLang;
            this.attributeHolder = attributeHolder;
        }

        @Override
        public int getLabelCount() {
            return titleLabelHolder.getLabelCount();
        }

        @Override
        public Label getLabelByIndex(int i) {
            return titleLabelHolder.getLabelByIndex(i);
        }

        @Override
        public Label getLabelByLang(Lang lang) {
            return titleLabelHolder.getLabelByLang(lang);
        }

        @Override
        public Phrases getPhrases() {
            return phrases;
        }

        @Override
        public URL getIcon() {
            return iconUrl;
        }

        @Override
        public URL getWebsite() {
            return websiteUrl;
        }

        @Override
        public Lang getDefaultLang() {
            return defaultLang;
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeHolder.getAttributeByKey(key);
        }

        @Override
        public int getAttributeCount() {
            return attributeHolder.getAttributeCount();
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeHolder.getAttributeByIndex(i);
        }

    }

}
