/* UtilLib - Copyright (c) 2011-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;


/**
 *
 * @author Vincent Calame
 */
public class ChangeLabelHolderBuilder {

    private final Set<Lang> removedLangSet = new LinkedHashSet<Lang>();
    private final Map<Lang, Label> labelMap = new LinkedHashMap<Lang, Label>();

    public ChangeLabelHolderBuilder() {

    }

    public void putLabel(Label label) {
        Lang lang = label.getLang();
        labelMap.put(lang, label);
        removedLangSet.remove(lang);
    }

    public Label putLabel(Lang lang, CharSequence charSequence) {
        return putLabel(lang, CleanedString.newInstance(charSequence));
    }

    public Label putLabel(Lang lang, CleanedString cleanedString) {
        if (cleanedString == null) {
            removedLangSet.add(lang);
            labelMap.remove(lang);
            return null;
        } else {
            Label label = LabelUtils.toLabel(lang, cleanedString);
            labelMap.put(lang, label);
            removedLangSet.remove(lang);
            return label;
        }
    }

    public int getAddedLabelCount() {
        return labelMap.size();
    }

    public Label getLabel(Lang lang) {
        return labelMap.get(lang);
    }

    public void putRemovedLang(Lang lang) {
        removedLangSet.add(lang);
        labelMap.remove(lang);
    }

    public void merge(ChangeLabelHolderBuilder otherChangeLabelHolderBuilder) {
        for (Lang otherLang : otherChangeLabelHolderBuilder.removedLangSet) {
            if (!labelMap.containsKey(otherLang)) {
                removedLangSet.add(otherLang);
            }
        }
        for (Map.Entry<Lang, Label> entry : otherChangeLabelHolderBuilder.labelMap.entrySet()) {
            Lang otherLang = entry.getKey();
            if (!labelMap.containsKey(otherLang)) {
                labelMap.put(otherLang, entry.getValue());
                removedLangSet.remove(otherLang);
            }
        }
    }

    public void clear() {
        removedLangSet.clear();
        labelMap.clear();
    }

    public ChangeLabelHolder toChangeLabelHolder() {
        Lang[] removedLangArray = removedLangSet.toArray(new Lang[removedLangSet.size()]);
        Label[] labelArray = labelMap.values().toArray(new Label[labelMap.size()]);
        return new InternalChangeLabelHolder(labelMap, labelArray, removedLangArray);
    }

    public LabelHolder toLabelHolder() {
        Label[] labelArray = labelMap.values().toArray(new Label[labelMap.size()]);
        return new InternalLabelHolder(labelMap, labelArray);
    }

    public Phrase toPhrase(String name) {
        Label[] labelArray = labelMap.values().toArray(new Label[labelMap.size()]);
        return new InternalPhrase(name, labelMap, labelArray);
    }


    private static class InternalChangeLabelHolder implements ChangeLabelHolder {

        private final Lang[] removedLangArray;
        private final Label[] labelArray;
        private final Map<Lang, Label> labelMap;

        private InternalChangeLabelHolder(Map<Lang, Label> labelMap, Label[] labelArray, Lang[] removedLangArray) {
            this.removedLangArray = removedLangArray;
            this.labelArray = labelArray;
            this.labelMap = labelMap;
        }

        @Override
        public int getLabelCount() {
            return labelArray.length;
        }

        @Override
        public Label getLabelByIndex(int i) {
            return labelArray[i];
        }

        @Override
        public Label getLabelByLang(Lang lang) {
            return labelMap.get(lang);
        }

        @Override
        public int getRemovedLangCount() {
            return removedLangArray.length;
        }

        @Override
        public Lang getRemovedLang(int i) {
            return removedLangArray[i];
        }

    }


    private static class InternalLabelHolder implements LabelHolder {

        private final Label[] labelArray;
        private final Map<Lang, Label> labelMap;

        private InternalLabelHolder(Map<Lang, Label> labelMap, Label[] labelArray) {
            this.labelArray = labelArray;
            this.labelMap = labelMap;
        }

        @Override
        public int getLabelCount() {
            return labelArray.length;
        }

        @Override
        public Label getLabelByIndex(int i) {
            return labelArray[i];
        }

        @Override
        public Label getLabelByLang(Lang lang) {
            return labelMap.get(lang);
        }

    }


    private static class InternalPhrase implements Phrase {

        private final String name;
        private final Label[] labelArray;
        private final Map<Lang, Label> labelMap;

        private InternalPhrase(String name, Map<Lang, Label> labelMap, Label[] labelArray) {
            this.name = name;
            this.labelArray = labelArray;
            this.labelMap = labelMap;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public int getLabelCount() {
            return labelArray.length;
        }

        @Override
        public Label getLabelByIndex(int i) {
            return labelArray[i];
        }

        @Override
        public Label getLabelByLang(Lang lang) {
            return labelMap.get(lang);
        }

    }

}
